// tslint:disable-next-line:no-reference
/// <reference path="../../node_modules/@types/jasmine/index.d.ts" />

import { Result } from "../../src/util/result";
import { RegimeUniforme } from "../../src/regime_uniforme";
import { cSnTrapez, ParamsSectionTrapez } from "../../src/section/section_trapez";
import { equalEpsilon, checkResult } from "../test_func";
import { precDist } from "../test_config";

describe("Class RegimeUniforme / section trapèze :", () => {
    describe("pas de débordement :", () => {
        it("LargeurFond should be 2.5", () => {
            const prms = new ParamsSectionTrapez(undefined, // largeur de fond
                0.56, // fruit
                0.8, // tirant d'eau
                40, //  Ks=Strickler
                1.988428,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);

            const ru = new RegimeUniforme(sect, false);

            checkResult(ru.Calc("LargeurFond", 0, precDist), 2.5);
        });

        it("Fruit should be 0.56", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                undefined, // fruit
                0.587, // tirant d'eau
                40, //  Ks=Strickler
                1.2,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Fruit", 0), 0.56);
        });

        it("Ks should be 24.14", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                0.56, // fruit
                0.8, // tirant d'eau
                undefined, //  Ks=Strickler
                1.2,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Ks", 1e-8), 24.14);
        });

        it("If should be 0.001", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                0.56, // fruit
                0.587, // tirant d'eau
                40, //  Ks=Strickler
                1.2,  //  Q=Débit
                undefined, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("If", 0.001), 0.001);
        });

        it("Q should be 1.2", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                0.56, //  fruit
                0.587, // tirant d'eau
                40, //  Ks=Strickler
                0,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Q", 0), 1.2);
        });

        it("Y should be 0.587", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                0.56, //  fruit
                undefined, // tirant d'eau
                40, //  Ks=Strickler
                1.2,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Y", 0), 0.587);
        });
    });

    describe("débordement :", () => {
        it("LargeurFond should be 0.03", () => {
            const prms = new ParamsSectionTrapez(undefined, // largeur de fond
                0.56, // fruit
                2, // tirant d'eau
                40, //  Ks=Strickler
                1.2,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );
            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect, false);

            checkResult(ru.Calc("LargeurFond", 0, precDist), 0.03);
        });

        it("Fruit should be 0.56", () => {
            const prms = new ParamsSectionTrapez(0.03, // largeur de fond
                undefined, // fruit
                2, // tirant d'eau
                40, //  Ks=Strickler
                1.2,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Fruit", 0), 0.56);
        });

        it("Ks should be 5.744", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                0.56, // fruit
                2, // tirant d'eau
                undefined, //  Ks=Strickler
                1.2,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );
            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Ks", 1e-8), 5.744);
        });

        it("If should be 0.001", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                0.56, // fruit
                2, // tirant d'eau
                40, //  Ks=Strickler
                8.356,  //  Q=Débit
                undefined, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );
            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("If", 0.001), 0.001);
        });

        it("Q should be 8.356", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                0.56, //  fruit
                2, // tirant d'eau
                40, //  Ks=Strickler
                undefined,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Q", 0), 8.356);
        });

        it("Y should be 0.587", () => {
            const prms = new ParamsSectionTrapez(2.5, // largeur de fond
                0.56, //  fruit
                undefined, // tirant d'eau
                40, //  Ks=Strickler
                1.2,  //  Q=Débit
                0.001, //  If=pente du fond
                precDist, // précision
                1, // YB= hauteur de berge
            );

            const sect = new cSnTrapez(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Y", 0), 0.587);
        });
    });
});
