import { cSnCirc, ParamsSectionCirc } from "../../src/section/section_circulaire";
import { precDist } from "../test_config";
import { checkResult } from "../test_func";

let paramSection: ParamsSectionCirc;
let sect: cSnCirc;

function createSection(prec: number): cSnCirc {
    paramSection = new ParamsSectionCirc(2, // diamètre
        0.8, // tirant d'eau
        40, //  Ks=Strickler
        1.2,  //  Q=Débit
        0.001, //  If=pente du fond
        prec, // précision
        1, // YB= hauteur de berge
    );

    return new cSnCirc(paramSection);
}

function createSectionDebordement(prec: number): cSnCirc {
    paramSection = new ParamsSectionCirc(2, // diamètre
        2, // tirant d'eau
        40, //  Ks=Strickler
        1.2,  //  Q=Débit
        0.001, //  If=pente du fond
        prec, // précision
        1, // YB= hauteur de berge
    );

    return new cSnCirc(paramSection);
}

describe("Section paramétrée circulaire : ", () => {
    beforeEach(() => {
        sect = createSection(precDist);
    });

    describe("fluvial / pas de débordement :", () => {
        // charge spécifique
        it("Hs should equal to 0.853", () => {
            checkResult(sect.Calc("Hs"), 0.853);
        });

        // charge critique
        it("Hsc should equal to 0.694", () => {
            checkResult(sect.Calc("Hsc"), 0.694);
        });

        // largeur au miroir
        it("B should equal to 1.959", () => {
            checkResult(sect.Calc("B"), 1.959);
        });

        // périmètre mouillé
        it("P should equal to 2.738", () => {
            checkResult(sect.Calc("P"), 2.738);
        });

        // surface mouillée
        it("S should equal to 1.173", () => {
            checkResult(sect.Calc("S"), 1.173);
        });

        // rayon hydraulique
        it("R should equal to 0.428", () => {
            checkResult(sect.Calc("R"), 0.428);
        });

        // vitesse moyenne
        it("V should equal to 1.023", () => {
            checkResult(sect.Calc("V"), 1.023);
        });

        // nombre de Froude
        it("Fr should equal to 0.422", () => {
            checkResult(sect.Calc("Fr"), 0.422);
        });

        // tirant d'eau critique
        it("Yc should equal to 0.512", () => {
            checkResult(sect.Calc("Yc"), 0.512);
        });

        // tirant d'eau normal
        it("Yn should equal to 0.976", () => {
            checkResult(sect.Calc("Yn"), 0.976);
        });

        // tirant d'eau fluvial
        it("Yf should equal to 0.8", () => {
            checkResult(sect.Calc("Yf"), 0.8);
        });

        // tirant d'eau torrentiel
        it("Yt should equal to 0.361", () => {
            checkResult(sect.Calc("Yt"), 0.361);
        });

        // tirant d'eau conjugué
        it("Yco should equal to 0.307", () => {
            checkResult(sect.Calc("Yco"), 0.307);
        });

        // perte de charge
        it("J should equal to 0.002", () => {
            // sect = createSection(0.00001);
            checkResult(sect.Calc("J"), 0.002);
        });

        // Variation linéaire de l'énergie spécifique
        it("I-J should equal to -0.00102", () => {
            sect = createSection(0.00001);
            checkResult(sect.Calc("I-J"), -0.00102);
        });

        // impulsion hydraulique
        it("Imp should equal to 5076.304", () => {
            checkResult(sect.Calc("Imp"), 5076.304);
        });

        // force tractrice (contrainte de cisaillement)
        it("Tau0 should equal to 8.505", () => {
            checkResult(sect.Calc("Tau0"), 8.505);
        });
    });
});

describe("Section paramétrée circulaire : ", () => {
    beforeEach(() => {
        sect = createSectionDebordement(precDist);
    });

    describe("fluvial / débordement :", () => {
        // charge spécifique
        it("Hs should equal to 2.006", () => {
            checkResult(sect.Calc("Hs"), 2.006);
        });

        // charge critique
        it("Hsc should equal to 0.694", () => {
            checkResult(sect.Calc("Hsc"), 0.694);
        });

        // largeur au miroir
        it("B should equal to 2", () => {
            checkResult(sect.Calc("B"), 2);
        });

        // périmètre mouillé
        it("P should equal to 5.142", () => {
            checkResult(sect.Calc("P"), 5.142);
        });

        // surface mouillée
        it("S should equal to 3.571", () => {
            checkResult(sect.Calc("S"), 3.571);
        });

        // rayon hydraulique
        it("R should equal to 0.694", () => {
            checkResult(sect.Calc("R"), 0.694);
        });

        // vitesse moyenne
        it("V should equal to 0.336", () => {
            checkResult(sect.Calc("V"), 0.336);
        });

        // nombre de Froude
        it("Fr should equal to 0.08", () => {
            checkResult(sect.Calc("Fr"), 0.08);
        });

        // tirant d'eau critique
        it("Yc should equal to 0.512", () => {
            checkResult(sect.Calc("Yc"), 0.512);
        });

        // tirant d'eau normal
        it("Yn should equal to 0.976", () => {
            checkResult(sect.Calc("Yn"), 0.976);
        });

        // tirant d'eau fluvial
        it("Yf should equal to 2", () => {
            checkResult(sect.Calc("Yf"), 2);
        });

        // tirant d'eau torrentiel
        it("Yt should equal to 0.232", () => {
            checkResult(sect.Calc("Yt"), 0.232);
        });

        // tirant d'eau conjugué
        it("Yco should equal to 0.24", () => {
            checkResult(sect.Calc("Yco"), 0.24);
        });

        // perte de charge
        it("J should equal to 0.0001", () => {
            sect = createSectionDebordement(0.00001);
            checkResult(sect.Calc("J"), 0.0001);
        });

        // Variation linéaire de l'énergie spécifique
        it("I-J should equal to 0.001", () => {
            sect = createSectionDebordement(0.00001);
            checkResult(sect.Calc("I-J"), 0.001);
        });

        // impulsion hydraulique
        it("Imp should equal to 6943.271", () => {
            checkResult(sect.Calc("Imp"), 6943.271);
        });

        // force tractrice (contrainte de cisaillement)
        it("Tau0 should equal to 0.782", () => {
            checkResult(sect.Calc("Tau0"), 0.782);
        });
    });
});
