import { cSnTrapez, ParamsSectionTrapez } from "../../src/section/section_trapez";
import { precDist } from "../test_config";
import { checkResult } from "../test_func";

let paramCnl: ParamsSectionTrapez;
let sect: cSnTrapez;

describe("Section paramétrée trapèze :", () => {
    beforeEach(() => {
        paramCnl = new ParamsSectionTrapez(2.5, // largeur de fond
            0.56, // fruit
            0.8, // tirant d'eau
            40, //  Ks=Strickler
            10,  //  Q=Débit
            0.001, //  If=pente du fond
            precDist, // précision
            1, // YB= hauteur de berge
        );

        sect = new cSnTrapez(paramCnl);
    });

    describe("torrentiel :", () => {
        // charge spécifique
        it("Hs should equal to 1.716", () => {
            checkResult(sect.Calc("Hs"), 1.716);
        });

        // charge critique
        it("Hsc should equal to 1.534", () => {
            checkResult(sect.Calc("Hsc"), 1.534);
        });

        // largeur au miroir
        it("B should equal to 3.396", () => {
            checkResult(sect.Calc("B"), 3.396);
        });

        // périmètre mouillé
        it("P should equal to 4.334", () => {
            checkResult(sect.Calc("P"), 4.334);
        });

        // surface mouillée
        it("S should equal to 2.358", () => {
            checkResult(sect.Calc("S"), 2.358);
        });

        // rayon hydraulique
        it("R should equal to 0.544", () => {
            checkResult(sect.Calc("R"), 0.544);
        });

        // vitesse moyenne
        it("V should equal to 4.24", () => {
            checkResult(sect.Calc("V"), 4.24);
        });

        // nombre de Froude
        it("Fr should equal to 1.625", () => {
            checkResult(sect.Calc("Fr"), 1.625);
        });

        // tirant d'eau critique
        it("Yc should equal to 1.074", () => {
            checkResult(sect.Calc("Yc"), 1.074);
        });

        // tirant d'eau normal
        it("Yn should equal to 2.275", () => {
            checkResult(sect.Calc("Yn"), 2.275);
        });

        // tirant d'eau fluvial
        it("Yf should equal to 1.502", () => {
            checkResult(sect.Calc("Yf"), 1.502);
        });

        // tirant d'eau torrentiel
        it("Yt should equal to 0.8", () => {
            checkResult(sect.Calc("Yt"), 0.8);
        });

        // tirant d'eau conjugué
        it("Yco should equal to 1.398", () => {
            checkResult(sect.Calc("Yco"), 1.398);
        });

        // perte de charge
        it("J should equal to 0.025", () => {
            // paramCnl.v.Prec = 0.00001;
            checkResult(sect.Calc("J"), 0.025);
        });

        // Variation linéaire de l'énergie spécifique
        it("I-J should equal to -0.024", () => {
            checkResult(sect.Calc("I-J"), -0.024);
        });

        // impulsion hydraulique
        it("Imp should equal to 51187.203", () => {
            checkResult(sect.Calc("Imp"), 51187.203);
        });

        // force tractrice (contrainte de cisaillement)
        it("Tau0 should equal to 135.020", () => {
            checkResult(sect.Calc("Tau0"), 135.020);
        });
    });
});
