import { Debug, Serie } from "./base";
import { Dichotomie } from "./dichotomie";
import { ComputeNode, ParamDefinition, ParamsEquation } from "./param";
import { Result } from "./util/result";

/**
 * Classe abstraite de Noeud de calcul : classe de base pour tous les calculs
 */
export abstract class Nub extends ComputeNode {
    private _dichoStartIntervalMaxSteps: number = 100;

    /*
     * paramétrage de la dichotomie
     */

    /**
     * étapes de recherche de l'intervalle de départ
     */
    set dichoStartIntervalMaxSteps(n: number) {
        this._dichoStartIntervalMaxSteps = n;
    }

    /**
     * Formule utilisée pour le calcul analytique (solution directe ou méthode de résolution spécifique)
     */
    public abstract Equation(sVarCalc: string): Result;

    /**
     * Calcul d'une équation quelque soit l'inconnue à calculer
     * @param sVarCalc nom de la variable à calculer
     * @param rInit valeur initiale de la variable à calculer dans le cas de la dichotomie
     * @param rPrec précision de calcul
     */
    public Calc(sVarCalc: string, rInit?: number, rPrec: number = 0.001): Result {
        if (rInit === undefined) {
            rInit = this._prms.map[sVarCalc].v;
        }
        if (this._prms.map[sVarCalc].isAnalytical()) {
            return this.Equation(sVarCalc);
        }

        const resSolve: Result = this.Solve(sVarCalc, rInit, rPrec);
        if (!resSolve.ok) {
            return resSolve;
        }
        const sAnalyticalPrm: string = this.getFirstAnalyticalParameter().symbol;
        this._prms.map[sVarCalc].v = resSolve.vCalc;
        const res: Result = this.Equation(sAnalyticalPrm);
        res.vCalc = resSolve.vCalc;
        return res;
    }

    public CalcSerie(svarCalc: string, serie: Serie): Result[] {
        /** @todo faire une boucle pour appeler this.Calc avec chaque valeur de serie.values
         *
         */
        // let results = [new (Result)];
        const results = [new Result(0)];
        return results;
    }

    /**
     * Résoud l'équation par une méthode numérique
     * @param sVarCalc nom de la variable à calculer
     * @param rInit valeur initiale de la variable à calculer dans le cas de la dichotomie
     * @param rPrec précision de calcul
     */
    private Solve(sVarCalc: string, rInit: number, rPrec: number): Result {
        const dicho: Dichotomie = new Dichotomie(this, sVarCalc, this.DBG);
        dicho.startIntervalMaxSteps = this._dichoStartIntervalMaxSteps;
        const target = this._prms.getFirstAnalyticalParameter();
        return dicho.Dichotomie(target.v, rPrec, rInit);
    }
}
