import { Result } from "../util/result";
import { ComputeNodeType, ParamDefinition, ParamDomain, ParamDomainValue, ParamCalculability, ParamsEquation } from "../param";
import { NumericalString } from "../util/numericalstring";
import { Nub } from "../nub";

export class PabPuissanceParams extends ParamsEquation {
    [key: string]: any; // pour pouvoir faire this['methode]();

    /** Chute entre bassins DH */
    private _DH: ParamDefinition;

    /** Débit Q */
    private _Q: ParamDefinition;

    /** Volume V */
    private _V: ParamDefinition;

    /** Puissance dissipée PV */
    private _Pv: ParamDefinition;

    constructor(rDH: number, rQ: number, rV: number, rPV: number = undefined) {
        super();
        this._DH = new ParamDefinition(ComputeNodeType.PabPuissance, 'DH', ParamDomainValue.POS, rDH);
        this._Q = new ParamDefinition(ComputeNodeType.PabPuissance, 'Q', ParamDomainValue.POS, rQ);
        this._V = new ParamDefinition(ComputeNodeType.PabPuissance, 'V', ParamDomainValue.POS, rV);
        this._Pv = new ParamDefinition(ComputeNodeType.PabPuissance, 'Pv', ParamDomainValue.POS, rPV);

        this.addParamDefinition(this._DH);
        this.addParamDefinition(this._Q);
        this.addParamDefinition(this._V);
        this.addParamDefinition(this._Pv);
    }

    get DH() {
        return this._DH;
    }

    get Q() {
        return this._Q;
    }

    get V() {
        return this._V;
    }

    get Pv() {
        return this._Pv;
    }
}
export class PabPuissance extends Nub {
    constructor(prms: PabPuissanceParams, dbg: boolean = false) {
        super(prms, dbg);
    }

    /**
     * paramètres castés au bon type
     */
    get prms(): PabPuissanceParams {
        return <PabPuissanceParams>this._prms;
    }

    /**
     * paramétrage de la calculabilité des paramètres
     */
    protected setParametersCalculability() {
        this.prms.DH.calculability = ParamCalculability.DICHO;
        this.prms.Q.calculability = ParamCalculability.DICHO;
        this.prms.V.calculability = ParamCalculability.DICHO;
        this.prms.Pv.calculability = ParamCalculability.EQUATION;
    }

    Equation(sVarCalc: string): Result {
        let v: number;

        switch (sVarCalc) {
            case "Pv":
                const ro: number = 1000;     // masse volumique de l'eau en kg/m3
                const g: number = 9.81;     // accélération de la gravité terrestre en m/s2.
                v = ro * g * this.prms.Q.v * this.prms.DH.v / this.prms.V.v;
                break;

            default:
                throw "PabPuissance.Equation() : invalid variable name " + sVarCalc;
        }

        return new Result(v);
    }
}
