import { ComputeNodeType, ParamsEquation } from "./param";
import { ConduiteDistribParams, ConduiteDistrib } from "./cond_distri";
import { LechaptCalmonParams, LechaptCalmon } from "./lechaptcalmon";
import { ParamsSection, acSection } from "./section/section_type";
import { ParamsSectionTrapez, cSnTrapez } from "./section/section_trapez";
import { ParamsSectionRectang, cSnRectang } from "./section/section_rectang";
import { ParamsSectionCirc, cSnCirc } from "./section/section_circulaire";
import { ParamsSectionPuiss, cSnPuiss } from "./section/section_puissance";
import { RegimeUniforme } from "./regime_uniforme";
import { CourbeRemous, CourbeRemousParams } from "./remous";
import { PabDimensionParams, PabDimension } from "./pab/pab_dimension";
import { PabPuissanceParams, PabPuissance } from "./pab/pab_puissance";

export class ComputeNodeParameters {
    private static _instance: ComputeNodeParameters;

    private constructor() { }

    public static getInstance() {
        if (ComputeNodeParameters._instance == undefined)
            ComputeNodeParameters._instance = new ComputeNodeParameters();
        return ComputeNodeParameters._instance;
    }

    public getComputeNodeParameters(type: ComputeNodeType): ParamsEquation {
        switch (type) {
            case ComputeNodeType.CondDistri:
                {
                    let cn = new ConduiteDistribParams(undefined, undefined, undefined, undefined, undefined);
                    let n = new ConduiteDistrib(cn); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.LechaptCalmon:
                {
                    let cn = new LechaptCalmonParams(undefined, undefined, undefined, undefined, undefined, undefined, undefined);
                    let n = new LechaptCalmon(cn); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.SectionTrapeze:
                {
                    let cn = new ParamsSectionTrapez(undefined, undefined, undefined, undefined,
                        undefined, undefined, undefined, undefined, undefined, undefined);
                    let n = new cSnTrapez(cn); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.SectionRectangle:
                {
                    let cn = new ParamsSectionRectang(undefined, undefined, undefined, undefined,
                        undefined, undefined, undefined, undefined);
                    let n = new cSnRectang(cn); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.SectionCercle:
                {
                    let cn = new ParamsSectionCirc(undefined, undefined, undefined, undefined,
                        undefined, undefined, undefined, undefined, undefined);
                    let n = new cSnCirc(cn); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.SectionPuissance:
                {
                    let cn = new ParamsSectionPuiss(undefined, undefined, undefined, undefined,
                        undefined, undefined, undefined, undefined, undefined, undefined);
                    let n = new cSnPuiss(cn); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.RegimeUniformeTrapeze:
                {
                    let cn = new ParamsSectionTrapez(undefined, undefined, undefined,
                        undefined, undefined, undefined, undefined, undefined,
                        ComputeNodeType.RegimeUniformeTrapeze,
                        ComputeNodeType.RegimeUniforme);
                    let n = new cSnTrapez(cn);
                    let ru = new RegimeUniforme(n); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.RegimeUniformeRectangle:
                {
                    let cn = new ParamsSectionRectang(undefined, undefined, undefined,
                        undefined, undefined, undefined, undefined,
                        ComputeNodeType.RegimeUniforme);
                    let n = new cSnRectang(cn);
                    let ru = new RegimeUniforme(n); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.RegimeUniformeCercle:
                {
                    let cn = new ParamsSectionCirc(undefined, undefined, undefined,
                        undefined, undefined, undefined, undefined,
                        ComputeNodeType.RegimeUniformeCercle,
                        ComputeNodeType.RegimeUniforme);
                    let n = new cSnCirc(cn);
                    let ru = new RegimeUniforme(n); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.RegimeUniformePuissance:
                {
                    let cn = new ParamsSectionPuiss(undefined, undefined, undefined,
                        undefined, undefined, undefined, undefined, undefined,
                        ComputeNodeType.RegimeUniformePuissance,
                        ComputeNodeType.RegimeUniforme);
                    let n = new cSnPuiss(cn);
                    let ru = new RegimeUniforme(n); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.CourbeRemousCercle:
                {
                    let cn = new ParamsSectionCirc(1, undefined, undefined, 1, undefined,
                        0.001, 1,
                        ComputeNodeType.CourbeRemousCercle,
                        ComputeNodeType.CourbeRemous);
                    let n = new cSnCirc(cn);
                    let crp = new CourbeRemousParams(n, undefined, undefined, undefined, undefined, undefined);
                    let ru = new CourbeRemous(crp); // pour initialiser la calculabilité des paramètres
                    return crp
                }

            case ComputeNodeType.CourbeRemousPuissance:
                {
                    let cn = new ParamsSectionPuiss(1, undefined, 1, undefined, 1,
                        undefined, 0.001, 1,
                        ComputeNodeType.CourbeRemousPuissance,
                        ComputeNodeType.CourbeRemous);
                    let n = new cSnPuiss(cn);
                    let crp = new CourbeRemousParams(n, undefined, undefined, undefined, undefined, undefined);
                    let ru = new CourbeRemous(crp); // pour initialiser la calculabilité des paramètres
                    return crp;
                }

            case ComputeNodeType.CourbeRemousRectangle:
                {
                    let cn = new ParamsSectionRectang(undefined, 1, undefined, 1, undefined,
                        0.001, 1,
                        ComputeNodeType.CourbeRemous);
                    let n = new cSnRectang(cn);
                    let crp = new CourbeRemousParams(n, undefined, undefined, undefined, undefined, undefined);
                    let ru = new CourbeRemous(crp); // pour initialiser la calculabilité des paramètres
                    return crp;
                }

            case ComputeNodeType.CourbeRemousTrapeze:
                {
                    let cn = new ParamsSectionTrapez(1, 1, undefined, undefined, 1,
                        undefined, 0.001, 1,
                        ComputeNodeType.CourbeRemousTrapeze,
                        ComputeNodeType.CourbeRemous);
                    let n = new cSnTrapez(cn);
                    let crp = new CourbeRemousParams(n, undefined, undefined, undefined, undefined, undefined);
                    let ru = new CourbeRemous(crp); // pour initialiser la calculabilité des paramètres
                    return crp;
                }

            case ComputeNodeType.PabDimensions:
                {
                    let cn = new PabDimensionParams(undefined, undefined, undefined);
                    let n = new PabDimension(cn); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            case ComputeNodeType.PabPuissance:
                {
                    let cn = new PabPuissanceParams(undefined, undefined, undefined);
                    let n = new PabPuissance(cn); // pour initialiser la calculabilité des paramètres
                    return cn;
                }

            default:
                throw "ComputeNodeParameters.getComputeNodeParameters() : noeud de calcul '" + ComputeNodeType[type] + "' non pris en charge";
        }
    }
}
