import { Result } from "./util/result";
import { Nub } from "./nub";
import { ParamsSection, acSection } from "./section/section_type";
import { ParamDefinition, ParamCalculability } from "./param"

export class RegimeUniforme extends Nub {
    private Sn: acSection;

    constructor(s: acSection, dbg: boolean = false) {
        super(s.prms, dbg);
        this.Sn = s;
    }

    protected setParametersCalculability() {
        this.prms.Q.calculability = ParamCalculability.EQUATION;
        this.prms.Y.calculability = ParamCalculability.EQUATION;
    }

    get prms(): ParamsSection {
        return <ParamsSection>this._prms;
    }

    /**
   * Calcul du débit en régime uniforme.
   * @return Débit en régime uniforme
   */
    Calc_Qn(): Result {
        this.Sn.Reset(true);
        if (this.Sn.prms.If.v <= 0)
            return new Result(0);

        // this.debug("RU: Calc_Qn : ");
        // this.debug(this.Sn.prms);

        // return this.Sn.prms.Ks.v * Math.pow(this.Sn.Calc("R", this.Sn.prms.Y.v), 2 / 3) * this.Sn.Calc("S", this.Sn.prms.Y.v) * Math.sqrt(this.Sn.prms.If.v);

        const rR: Result = this.Sn.Calc("R", this.Sn.prms.Y.v);
        if (!rR)
            return rR;

        const rS: Result = this.Sn.Calc("S", this.Sn.prms.Y.v)
        if (!rS)
            return rS;

        const v = this.Sn.prms.Ks.v * Math.pow(rR.vCalc, 2 / 3) * rS.vCalc * Math.sqrt(this.Sn.prms.If.v);
        return new Result(v);
    }

    Equation(sVarCalc: string): Result {
        this.debug("RU: Equation(" + sVarCalc + ")")

        switch (sVarCalc) {
            case 'Y':
                return this.Sn.Calc("Yn");

            case 'Q':
                return this.Calc_Qn();

            default:
                throw "RegimeUniforme.Equation() : invalid variable name " + sVarCalc;
        }
    }
}
