import { acSection, ParamsSection } from "./section_type";
import { ComputeNodeType } from "../param";
import { Result } from "../util/result";

export class ParamsSectionRectang extends ParamsSection {
        constructor(rY: number, rLargeurFond: number, rKs: number, rQ: number, rIf: number, rPrec: number, rYB: number,
                defaultNodeType: ComputeNodeType = ComputeNodeType.SectionParametree) {
                super(rY,
                        rLargeurFond, // LargeurBerge=LargeurFond
                        rKs, rQ, rIf, rPrec, rYB, defaultNodeType);
        }
}

/**
 * Calculs de la section rectangulaire
 */
export class cSnRectang extends acSection {
        constructor(prms: ParamsSectionRectang, dbg: boolean = false) {
                super(prms, dbg);
        }

        get prms(): ParamsSectionRectang {
                return <ParamsSectionRectang>this._prms;
        }

        /**
         * Calcul du périmètre mouillé
         * @return Périmètre mouillé (m)
         */
        protected Calc_P(): Result {
                // return this.prms.LargeurBerge.v + super.Calc_P_Debordement(this.prms.Y.v);
                const rPDeb: Result = super.Calc_P_Debordement(this.prms.Y.v);
                if (!rPDeb.ok)
                        return rPDeb;

                const v = this.prms.LargeurBerge.v + rPDeb.vCalc;
                return new Result(v);
        }

        protected Calc_dP(): Result {
                return super.Calc_dP_Debordement();
        }

        protected Calc_B(): Result {
                return super.Calc_B_Debordement();
        }

        protected Calc_dB(): Result {
                return super.Calc_dB_Debordement();
        }

        protected Calc_S(): Result {
                return super.Calc_S_Debordement(this.prms.Y.v);
        }

        /**
         * Calcul du tirant d'eau conjugué avec la formule analytique pour la section rectangulaire
         * @return tirant d'eau conjugué
         */
        protected CalcYco(): Result {
                // return this.prms.Y.v * (Math.sqrt(1 + 8 * Math.pow(this.Calc("Fr"), 2)) - 1) / 2;

                const rFR: Result = this.Calc("Fr");
                if (!rFR.ok)
                        return rFR;

                const v = this.prms.Y.v * (Math.sqrt(1 + 8 * Math.pow(rFR.vCalc, 2)) - 1) / 2;
                return new Result(v);
        }
}