import { RectangularStructureParams } from "./rectangular_structure_params";
import { Structure } from "./structure";
import { StructureCem88d } from "./structure_cem88d";
import { StructureCem88v } from "./structure_cem88v";
import { StructureCunge80 } from "./structure_cunge80";
import { StructureOrificeFree } from "./structure_orifice_free";
import { StructureOrificeSubmerged } from "./structure_orifice_submerged";
import { StructureWeirFree } from "./structure_weir_free";

export enum StructureType {
    Cem88d,
    Cem88v,
    Cunge80,
    OrificeFree,
    OrificeSubmerged,
    WeirFree
}

export function CreateStructure(structureType: StructureType): Structure {
    const structPrms: RectangularStructureParams = new RectangularStructureParams(
        0,  // Q
        100,        // ZDV
        102,        // Z1
        101.5,      // Z2
        2,          // L
        0.6,        // Cd
        0.5         // W
    );
    switch (structureType) {
        case StructureType.Cem88d:
            return new StructureCem88d(structPrms);
        case StructureType.Cem88v:
            return new StructureCem88v(structPrms);
        case StructureType.Cunge80:
            return new StructureCunge80(structPrms);
        case StructureType.OrificeFree:
            return new StructureOrificeFree(structPrms);
        case StructureType.OrificeSubmerged:
            return new StructureOrificeSubmerged(structPrms);
        case StructureType.WeirFree:
            const st: StructureWeirFree = new StructureWeirFree(structPrms);
            st.prms.Cd.v = 0.4;
            return st;
    }
}
