import { Result } from "../util/result";
import { RectangularStructure } from "./rectangular_structure";
import { RectangularStructureParams } from "./rectangular_structure_params";
import { Structure, StructureFlowMode, StructureFlowRegime } from "./structure";

export { RectangularStructureParams };

/**
 * Equation classique orifice dénoyé
 */
export class StructureOrificeFree extends RectangularStructure {
    /**
     * Calcul du débit avec l'équation classique d'un orifice dénoyé
     * @param sVarCalc Variable à calculer (doit être égale à Q ici)
     */
    public Equation(sVarCalc: string): Result {
        Structure.CheckEquation(sVarCalc);
        const data = this.getResultData();

        const v = this.prms.Cd.v * Math.min(this.prms.W.v, this.prms.h1.v) * this.prms.L.v
            * Structure.R2G * Math.sqrt(this.prms.h1.v);

        return new Result(v, data);
    }

    protected getFlowRegime() {
        return StructureFlowRegime.FREE;
    }
}
