export class DefinedValue<T> {
    private _value: T;

    constructor(v: T = undefined) {
        this._value = v;
    }

    get isDefined() {
        return this._value != undefined;
    }

    undefine() {
        this._value = undefined;
    }

    get uncheckedValue() {
        return this._value;
    }

    /**
     * fonction nécessitée par le fait qu'on ne peut pas appeller les accesseurs d'un parent
     * ou que si l'un des 2 accesseurs est surchargé, il faut surcharger l'autre
     * (et donc pour pouvoir éventuellement appeler l'accesseur parent)
     */
    protected getValue(): T {
        if (this._value == undefined)
            throw "undefined value";

        return this._value;
    }

    /**
     * fonction nécessitée par le fait qu'on ne peut pas appeller les accesseurs d'un parent
     * ou que si l'un des 2 accesseurs est surchargé, il faut surcharger l'autre
     * (et donc pour pouvoir éventuellement appeler l'accesseur parent)
     */
    protected setValue(v: T) {
        this._value = v;
    }

    get value() {
        return this.getValue();
    }

    set value(v: T) {
        this.setValue(v);
    }

    toString(): string {
        if (this.isDefined)
            return String(this._value);

        return "undefined";
    }
}

export class DefinedBoolean extends DefinedValue<boolean>{
}

export class DefinedNumber extends DefinedValue<number>{
}

export class DefinedString extends DefinedValue<string>{
    constructor(v: any = undefined) {
        super(v == undefined ? v : String(v));
    }
}
