import { Pair } from "./pair";
import { Message, MessageCode } from "./message";


/**
 * couple de valeurs ordonnées
 */
export class Interval extends Pair {
    constructor(bound1: number, bound2: number) {
        super(bound1, bound2);
    }

    checkValue(v: number) {
        if (v == undefined) {
            let e = new Message(MessageCode.ERROR_INTERVAL_UNDEF);
            throw e;
        }

        if (!this.intervalHasValue(v)) {
            let e = new Message(MessageCode.ERROR_INTERVAL_OUTSIDE);
            e.extraVar["value"] = v;
            e.extraVar["interval"] = this.toString();
            throw e;
        }
    }

    get length(): number {
        return this.max - this.min;
    }

    intersect(i: Interval): Interval {
        let min = Math.max(this.min, i.min);
        let max = Math.min(this.max, i.max);
        return new Interval(min, max);
    }

    setBounds(i: Pair) {
        this.setPair(i);
    }
}
