export enum MessageCode {
    /**
     * pas de pb !
     */
    ERROR_OK = 0,

    /**
     * la dichotomie n'a pas pu trouver automatiquement d'intervalle de départ
     * car la valeur initiale de la variable est trop haute
     */
    ERROR_DICHO_INITVALUE_HIGH = -1,

    /**
     * la dichotomie n'a pas pu trouver automatiquement d'intervalle de départ
     * car la valeur initiale de la variable est trop basse
     */
    ERROR_DICHO_INITVALUE_LOW = -2,

    /**
     * la dichotomie n'a pas pu trouver automatiquement d'intervalle de départ
     * car la valeur cible de la fonction n'existe pas pour des valeurs de la
     * variable dans son domaine de définition, cad il n'existe pas de solution
     */
    ERROR_DICHO_INIT_DOMAIN = -3,

    /**
     * la dichotomie n'a pas pu converger
     */
    ERROR_DICHO_CONVERGE = -4,

    /**
     * le pas pour la recherche de l'intervalle de départ est =0
     */
    ERROR_DICHO_NULL_STEP = -5,

    /**
     * l'augmentation du pas pour la recherche de l'intervalle de départ est incorrecte (=0)
     */
    ERROR_DICHO_INVALID_STEP_GROWTH = -6,

    /**
     * impossible de déterminer le sens de variation de la fonction
     */
    ERROR_DICHO_FUNCTION_VARIATION = -7,

    /**
     * les bornes de l'intervalle d'un ParamDomain sont incorrectes
     */
    ERROR_PARAMDOMAIN_INTERVAL_BOUNDS = -100,

    /**
     * la valeur du ParamDomain est incorrecte
     */
    ERROR_PARAMDOMAIN_INVALID = -101,

    /**
     * la calculabilité d'un ParamDefinition est non définie
     */
    ERROR_PARAMDEF_CALC_UNDEFINED = -200,

    /**
     * la valeur d'un ParamDefinition est non définie
     */
    ERROR_PARAMDEF_VALUE_UNDEFINED = -201,

    /**
     * la valeur d'un ParamDefinition ne peut pas être changée
     */
    ERROR_PARAMDEF_VALUE_FIXED = -202,

    /**
     * la valeur d'un ParamDefinition ne peut pas être > 0
     */
    ERROR_PARAMDEF_VALUE_POS = -203,

    /**
     * la valeur d'un ParamDefinition ne peut pas être >= 0
     */
    ERROR_PARAMDEF_VALUE_POSNULL = -204,

    /**
     * la valeur d'un ParamDefinition ne peut pas être = 0
     */
    ERROR_PARAMDEF_VALUE_NULL = -205,

    /**
     * la valeur d'un ParamDefinition est en dehors de son intervalle autorisé
     */
    ERROR_PARAMDEF_VALUE_INTERVAL = -206,

    /**
     * la valeur passée à une méthode de la classe Interval est undefined
     */
    ERROR_INTERVAL_UNDEF = -300,

    /**
     * la valeur passée à une méthode de la classe Interval est hors de l'intervalle défini
     */
    ERROR_INTERVAL_OUTSIDE = -301,

    /**
     * internationalisation : langue non prise en charge
     */
    ERROR_LANG_UNSUPPORTED = -400,

    /**
     * courbes de remous : Arrêt du calcul : hauteur critique atteinte à l'abscisse x
     */
    WARNING_REMOUS_ARRET_CRITIQUE = -500,

    /**
     * courbe de remous : Condition limite aval >= Hauteur critique : calcul de la partie fluviale à partir de l'aval
     */
    INFO_REMOUS_CALCUL_FLUVIAL = -501,

    /**
     * courbe de remous : Condition limite amont <= Hauteur critique : calcul de la partie torrentielle à partir de l'amont
     */
    INFO_REMOUS_CALCUL_TORRENTIEL = -502,

    /**
     * courbe de remous : ressaut hydraulique détecté à l'amont/aval de l'abscisse x
     */
    INFO_REMOUS_RESSAUT_DEHORS = -503,

    /**
     * courbe de remous : Largeur au niveau des berges
     */
    INFO_REMOUS_LARGEUR_BERGE = -504,

    /**
     * courbe de remous : Tirant d'eau critique
     */
    INFO_REMOUS_H_CRITIQUE = -505,

    /**
     * courbe de remous : Tirant d'eau normal
     */
    INFO_REMOUS_H_NORMALE = -506,

    /**
     * courbe de remous : Ressaut hydraulique détecté entre les abscisses Xmin et Xmax m
     */
    INFO_REMOUS_RESSAUT_HYDRO = -507,

    /**
     * courbe de remous : La pente de la ligne d'eau est trop forte à l'abscisse x m
     */
    ERROR_REMOUS_PENTE_FORTE = -508,

    /**
     * courbe de remous : Condition limite aval < Hauteur critique : pas de calcul possible depuis l'aval
     */
    ERROR_REMOUS_PAS_CALCUL_DEPUIS_AVAL = -509,

    /**
     * courbe de remous : Condition limite amont > Hauteur critique : pas de calcul possible depuis l'amont
     */
    ERROR_REMOUS_PAS_CALCUL_DEPUIS_AMONT = -510,

    /**
     * section : Non convergence du calcul de la hauteur critique (Méthode de Newton)
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HCRITIQUE = -600,

    /**
     * section : Non convergence du calcul de la hauteur normale (Méthode de Newton)
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HNORMALE = -601,

    /**
     * section : Non convergence du calcul de la hauteur conjuguée (Méthode de Newton)
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HCONJUG = -602,

    /**
     * section : Non convergence du calcul de la hauteur correspondante (Méthode de Newton) pour le calcul de la hauteur fluviale
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HFLU = -603,

    /**
     * section : Non convergence du calcul de la hauteur correspondante (Méthode de Newton) pour le calcul de la hauteur torrentielle
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HTOR = -604,

    /**
     * section : La pente est négative ou nulle, la hauteur normale est infinie
     */
    ERROR_SECTION_PENTE_NEG_NULLE_HNORMALE_INF = -605,

    /**
     * section : calcul impossible à cause d'un périmètre nul
     */
    ERROR_SECTION_PERIMETRE_NUL = -606,

    /**
     * section : calcul impossible à cause d'un rayon nul
     */
    ERROR_SECTION_RAYON_NUL = -607,

    /**
     * section : calcul impossible à cause d'une surface nulle
     */
    ERROR_SECTION_SURFACE_NULLE = -608,

    /**
     * newton : pas de convergence
     */
    ERROR_NEWTON_NON_CONVERGENCE = -700,

    /**
     * newton : dérivée nulle
     */
    ERROR_NEWTON_DERIVEE_NULLE = -701,
}

/**
 * niveau de criticité du message
 */
export enum MessageSeverity {
    ERROR, WARNING, INFO
}

/**
 * Résultat de calcul comprenant la valeur du résultat et des calculs annexes (flag, calculs intermédiaires...)
 */
export class Message {
    /**
     * code du message
     */
    private _code: MessageCode;

    /** Variables intermédiaires, flags d'erreur */
    public extraVar: { [key: string]: any };

    constructor(c: MessageCode) {
        this._code = c;
        this.extraVar = {}
    }

    get code() { return this._code; }

    /**
     * retourne le niveau de criticité (erreur, warning, info) du message
     */
    public getSeverity(): MessageSeverity {
        let m: string = MessageCode[this._code];
        let prefix: string = m.split("_")[0];
        for (let s in MessageSeverity) // pas nécessaire, mais permet d'avoir un contrôle de validité
            if (MessageSeverity[s] == prefix)
                return eval("MessageSeverity." + prefix);
        throw "Message.getSeverity() : valeur de code '" + this._code + "' invalide";
    }

    public toString(): string {
        return MessageCode[this._code] + " " + JSON.stringify(this.extraVar);
    }
}
